@@ -12,10 +12,6 @@
 #include "table-dewindow.h"
 #include "mp3-cos-tables.h"
 
-#ifdef USE_LIBOIL
-#include <liboil/liboil.h>
-#endif
-
 #define OPT_SYNTH
 
 void
@@ -46,7 +42,7 @@ III_subband_synthesis (mp3tl * tl, frame_params * fr_ps,
 
 #define INV_SQRT_2 (7.071067811865474617150084668537e-01f)
 
-#if defined(USE_ARM_NEON) && defined(__ARM_NEON__) && !defined(__SOFTFP__)
+#if defined(USE_ARM_NEON)
 static const __CACHE_LINE_DECL_ALIGN(float dct8_k[8]) = {
   INV_SQRT_2, 0.0f, 5.4119610014619701222e-01f, 1.3065629648763763537e+00f,
   5.0979557910415917998e-01f, 6.0134488693504528634e-01f,
@@ -148,7 +144,7 @@ MPG_DCT_8 (gfloat in[8], gfloat out[8])
 }
 #endif
 
-#if defined(USE_ARM_NEON) && defined(__ARM_NEON__) && !defined(__SOFTFP__)
+#if defined(USE_ARM_NEON)
 
 static const __CACHE_LINE_DECL_ALIGN(float dct16_k[8]) = {
   5.0241928618815567820e-01f, 5.2249861493968885462e-01f,
@@ -290,7 +286,7 @@ MPG_DCT_32 (gfloat in[32], gfloat out[32])
   out[31] = odd_out[15];
 }
 
-#if defined(USE_ARM_NEON) && defined(__ARM_NEON__) && !defined(__SOFTFP__)
+#if defined(USE_ARM_NEON)
 
 #define WIN_MAC \
   "  vld1.32     {q0-q1}, [r3,:128], r5         \n\t" /* read win */           \
@@ -351,7 +347,7 @@ mp3_dewindow_output (gfloat *uvec, short *samples, gfloat* window)
   );
 }
 
-#elif defined(USE_ARM_VFP) && defined(__VFP_FP__) && !defined(__SOFTFP__)
+#elif defined(USE_ARM_VFP)
 #define CONVERT_TO_INTEGER(v,d)                                               \
     "  fcmpezs     " v "                 \n\t"                                \
     "  fmstat                            \n\t"                                \
@@ -435,21 +431,10 @@ mp3_dewindow_output (gfloat *u_vec, short *samples, gfloat* window)
   gfloat *u_vec0;
 
   /* dewindowing */
-#ifdef USE_LIBOIL
-  oil_multiply_f32 (u_vec, u_vec, window, HAN_SIZE);
-#else
   for (i = 0; i < HAN_SIZE; i++)
     u_vec[i] *= dewindow[i];
-#endif
 
   /* Now calculate 32 samples */
-#ifdef USE_LIBOIL
-  u_vec0 = u_vec + 32;
-  for (i = 0; i < 15; i++) {
-    oil_add_f32 (u_vec, u_vec, u_vec0, 32);
-    u_vec0 += 32;
-  }
-#else
   for (i = 0; i < 32; i++) {
     gfloat sum;
     u_vec0 = u_vec + i;
@@ -470,7 +455,6 @@ mp3_dewindow_output (gfloat *u_vec, short *samples, gfloat* window)
     sum += u_vec0[15 << 5];
     u_vec0[0] += sum;
   }
-#endif
 
   for (i = 0; i < 32; i++) {
     gfloat sample = u_vec[i];
@@ -493,7 +477,7 @@ mp3_dewindow_output (gfloat *u_vec, short *samples, gfloat* window)
 }
 #endif
 
-#if defined(USE_ARM_NEON) && defined(__ARM_NEON__) && !defined(__SOFTFP__)
+#if defined(USE_ARM_NEON)
 STATIC_INLINE void
 build_uvec (gfloat *u_vec, gfloat *cur_synbuf, gint k)
 {
@@ -536,7 +520,7 @@ build_uvec (gfloat *u_vec, gfloat *cur_synbuf, gint k)
       "  add         r3, %[u_vec], #48*4            \n\t"
       "  vld1.64     {q1-q2}, [r4,:128]!            \n\t"
       "  vld1.64     {q3-q4}, [r4,:128]!            \n\t"
-      "  vldr.32     s2, [r4,:64]                   \n\t"
+      "  vldr.32     s2, [r4]                       \n\t"
       "  vneg.f32    q1, q1                         \n\t"
       "  vneg.f32    q2, q2                         \n\t"
       "  vneg.f32    q3, q3                         \n\t"
