@@ -14,9 +14,6 @@
 #endif
 
 #include <gst/gst.h>
-#ifdef USE_LIBOIL
-#include <liboil/liboil.h>
-#endif
 
 #include "mp3tl-priv.h"
 #include "decode.h"
@@ -34,7 +31,6 @@ GST_DEBUG_CATEGORY_EXTERN (flump3debug);
 #define MXOFF   250
 
 static const HUFFBITS dmask = (guint32) (1) << (sizeof (HUFFBITS) * 8 - 1);
-static const guint hs = sizeof (HUFFBITS) * 8;
 
 /************************* Layer III routines **********************/
 
@@ -168,58 +164,58 @@ const gint slen[2][16] = {
 struct
 {
   gint l[23];
-  gint s[14];
+  gint s[15];
 } const sfBandIndex[] = {
   /* MPEG-1 */
   {
     /* 44.1 khz */
     { 0, 4, 8, 12, 16, 20, 24, 30, 36, 44, 52, 62, 74, 90, 110, 134, 162, 196,
       238, 288, 342, 418, 576},
-    { 0, 4, 8, 12, 16, 22, 30, 40, 52, 66, 84, 106, 136, 192 }
+    { 0, 4, 8, 12, 16, 22, 30, 40, 52, 66, 84, 106, 136, 192, 192 }
   }, {
     /* 48khz */
     { 0, 4, 8, 12, 16, 20, 24, 30, 36, 42, 50, 60, 72, 88, 106, 128, 156, 190,
       230, 276, 330, 384, 576},
-    { 0, 4, 8, 12, 16, 22, 28, 38, 50, 64, 80, 100, 126, 192 }
+    { 0, 4, 8, 12, 16, 22, 28, 38, 50, 64, 80, 100, 126, 192, 192 }
   }, {
     /* 32khz */
     { 0, 4, 8, 12, 16, 20, 24, 30, 36, 44, 54, 66, 82, 102, 126, 156, 194, 240,
       296, 364, 448, 550, 576 },
-    { 0, 4, 8, 12, 16, 22, 30, 42, 58, 78, 104, 138, 180, 192 }
+    { 0, 4, 8, 12, 16, 22, 30, 42, 58, 78, 104, 138, 180, 192, 192 }
   },
   /* MPEG-2 */
   {
     /* 22.05 khz */
     { 0, 6, 12, 18, 24, 30, 36, 44, 54, 66, 80, 96, 116, 140, 168, 200, 238,
       284, 336, 396, 464, 522, 576 },
-    { 0, 4, 8, 12, 18, 24, 32, 42, 56, 74, 100, 132, 174, 192 }
+    { 0, 4, 8, 12, 18, 24, 32, 42, 56, 74, 100, 132, 174, 192, 192 }
   }, {
     /* 24khz */
     { 0, 6, 12, 18, 24, 30, 36, 44, 54, 66, 80, 96, 114, 136, 162, 194, 232,
       278, 330, 394, 464, 540, 576 },
-    { 0, 4, 8, 12, 18, 26, 36, 48, 62, 80, 104, 136, 180, 192 }
+    { 0, 4, 8, 12, 18, 26, 36, 48, 62, 80, 104, 136, 180, 192, 192 }
   }, {
     /* 16 khz */
     { 0, 6, 12, 18, 24, 30, 36, 44, 54, 66, 80, 96, 116, 140, 168, 200, 238,
       284, 336, 396, 464, 522, 576 },
-    { 0, 4, 8, 12, 18, 26, 36, 48, 62, 80, 104, 134, 174, 192 }
+    { 0, 4, 8, 12, 18, 26, 36, 48, 62, 80, 104, 134, 174, 192, 192 }
   },
   /* MPEG-2.5 */
   {
     /* 11025 */
     { 0, 6, 12, 18, 24, 30, 36, 44, 54, 66, 80, 96, 116, 140, 168, 200, 238,
       284, 336, 396, 464, 522, 576 },
-    { 0, 4, 8, 12, 18, 26, 36, 48, 62, 80, 104, 134, 174, 192 }
+    { 0, 4, 8, 12, 18, 26, 36, 48, 62, 80, 104, 134, 174, 192, 192 }
   }, {
     /* 12khz */
     { 0, 6, 12, 18, 24, 30, 36, 44, 54, 66, 80, 96, 116, 140, 168, 200, 238,
       284, 336, 396, 464, 522, 576 },
-    { 0, 4, 8, 12, 18, 26, 36, 48, 62, 80, 104, 134, 174, 192 }
+    { 0, 4, 8, 12, 18, 26, 36, 48, 62, 80, 104, 134, 174, 192, 192 }
   }, {
     /* 8khz */
     { 0, 12, 24, 36, 48, 60, 72, 88, 108, 132, 160, 192, 232, 280, 336, 400,
       476, 566, 568, 570, 572, 574, 576 },
-    { 0, 8, 16, 24, 36, 52, 72, 96, 124, 160, 162, 164, 166, 192 }
+    { 0, 8, 16, 24, 36, 52, 72, 96, 124, 160, 162, 164, 166, 192, 192 }
   }
 };
 
@@ -556,21 +552,18 @@ III_huffman_decode (gint is[SBLIMIT][SSLIMIT], III_side_info_t * si,
   sfreq = sfb_offset[fr_ps->header.version] + fr_ps->header.srate_idx;
 
   /* Find region boundary for short block case. */
-  if (!gi->window_switching_flag || (gi->block_type != 2 &&
-      !(fr_ps->header.version == MPEG_VERSION_2_5)))   {
-    region1Start = sfBandIndex[sfreq].l[gi->region0_count + 1]; /* MI */
-    region2Start = sfBandIndex[sfreq].l[gi->region0_count + gi->region1_count + 2];     /* MI */
-  } else {                      /* Find region boundary for long block case. */
-    if (gi->block_type == 2) {
-      if (fr_ps->header.version == MPEG_VERSION_2_5) {
-        region1Start = 72;          /* sfb[9/3]*3=36 */
-      } else {
-        region1Start = 36;          /* sfb[9/3]*3=36 */
-      }
-    } else {
-      region1Start = 108;          /* sfb[9/3]*3=36 */
+  if ((gi->window_switching_flag) && (gi->block_type == 2)) {
+    /* Region2. */
+    if (fr_ps->header.version == MPEG_VERSION_2_5
+          && fr_ps->header.srate_idx == 2) {
+      region1Start = 72;
+	} else {
+      region1Start = 36;          /* sfb[9/3]*3=36 */
     }
     region2Start = 576;         /* No Region2 for short block case. */
+  } else {                      /* Find region boundary for long block case. */
+    region1Start = sfBandIndex[sfreq].l[gi->region0_count + 1]; /* MI */
+    region2Start = sfBandIndex[sfreq].l[gi->region0_count + gi->region1_count + 2];     /* MI */
   }
 
   /* Read bigvalues area. */
@@ -1203,7 +1196,7 @@ inv_mdct_s (gfloat invec[6], gfloat outvec[12])
   outvec[11] = -h[2];
 }
 
-#if defined(USE_ARM_VFP) && defined(__VFP_FP__) && !defined(__SOFTFP__)
+#if defined(USE_ARM_VFP)
 static inline void
 calculate_H (gfloat * H, gfloat * invec)
 {
@@ -1244,10 +1237,8 @@ inv_mdct_l (gfloat invec[18], gfloat outvec[36])
   int i;
   gfloat H[17], h[18], even[9], odd[9], even_idct[9], odd_idct[9], *tmp;
 
-#if defined(USE_ARM_VFP) && defined(__VFP_FP__) && !defined(__SOFTFP__)
+#if defined(USE_ARM_VFP)
   calculate_H (H, invec);
-#elif defined(USE_LIBOIL)
-  oil_add_f32 (H, invec, invec + 1, 17);
 #else
   for (i = 0; i < 17; i++)
     H[i] = invec[i] + invec[i + 1];
@@ -1366,7 +1357,7 @@ imdct_9pt (gfloat invec[9], gfloat outvec[9])
   }
 }
 
-#if defined(USE_ARM_VFP) && defined(__VFP_FP__) && !defined(__SOFTFP__)
+#if defined(USE_ARM_VFP)
 static inline void
 dewindow_block_type_2 (gfloat * dst, gfloat * src,  gfloat * win )
 {
@@ -1468,7 +1459,7 @@ inv_mdct (gfloat in[18], gfloat out[36], gint block_type)
 
       /* The three short blocks must be windowed and overlapped added
        * with each other */
-#if defined(USE_ARM_VFP) && defined(__VFP_FP__) && !defined(__SOFTFP__)
+#if defined(USE_ARM_VFP)
       dewindow_block_type_2 (tmpptr, tmp, (gfloat *)win[2]);
 #else
       for (j = 0; j < 12; j++) {
@@ -1480,10 +1471,8 @@ inv_mdct (gfloat in[18], gfloat out[36], gint block_type)
     inv_mdct_l (in, out);
 
     /* Window the imdct result */
-#if defined(USE_ARM_VFP) && defined(__VFP_FP__) && !defined(__SOFTFP__)
+#if defined(USE_ARM_VFP)
     dewindow_block_type_other (out, out, (gfloat *) win[block_type]);
-#elif defined(USE_LIBOIL)
-    oil_multiply_f32 (out, out, win[block_type], 36);
 #else
     for (i = 0; i < 36; i++)
       out[i] = out[i] * win[block_type][i];
@@ -1543,7 +1532,7 @@ init_hybrid (mp3cimpl_info * c_impl)
         c_impl->prevblck[i][j][k] = 0.0;
 }
 
-#if defined(USE_ARM_VFP) && defined(__VFP_FP__) && !defined(__SOFTFP__)
+#if defined(USE_ARM_VFP)
 static inline void
 overlap_addition (gfloat * dst, gfloat * src,  gfloat * prev )
 {
@@ -1608,12 +1597,8 @@ III_hybrid (gfloat fsIn[SSLIMIT], gfloat tsOut[SSLIMIT], int sb, int ch,
   inv_mdct (fsIn, rawout, i);
 
   /* overlap addition */
-#if defined(USE_ARM_VFP) && defined(__VFP_FP__) && !defined(__SOFTFP__)
+#if defined(USE_ARM_VFP)
   overlap_addition (tsOut, rawout, (gfloat *) tl->c_impl.prevblck[ch][sb]);
-#elif defined(USE_LIBOIL)
-  oil_add_f32 (tsOut, rawout, (gfloat *) tl->c_impl.prevblck[ch][sb], SSLIMIT);
-  oil_memcpy (tl->c_impl.prevblck[ch][sb], rawout + 18,
-      SSLIMIT * sizeof (gfloat));
 #else
   for (i = 0; i < SSLIMIT; i++) {
     tsOut[i] = (gfloat) (rawout[i] + tl->c_impl.prevblck[ch][sb][i]);
@@ -1644,8 +1629,10 @@ c_decode_mp3 (mp3tl * tl)
   III_side_info_t III_side_info;
   huffdec_bitbuf *bb;
   guint gr, ch, sb;
+  gint diff;
   fr_header *hdr;
   guint8 side_info[32];         /* At most 32 bytes side info for MPEG-1 stereo */
+  gboolean MainDataOK;
 
   hdr = &tl->fr_ps.header;
   bb = &tl->c_impl.bb;
@@ -1664,26 +1651,34 @@ c_decode_mp3 (mp3tl * tl)
   if (bs_bits_avail (tl->bs) < hdr->main_slots * 8)
     return MP3TL_ERR_NEED_DATA;
 
-  /* Check that we have enough main_data between the bit reservoir
-   * and the incoming data */
-  if (tl->c_impl.main_data_end - III_side_info.main_data_begin < 0) {
-    /* Usually happens after a seek. We can't decode this frame */
-    GST_LOG ("Not enough main data available to decode frame");
-    return MP3TL_ERR_BAD_FRAME;
+  /* Verify that sufficient main_data was extracted from */
+  /* the previous sync interval */
+  diff = tl->c_impl.main_data_end - III_side_info.main_data_begin;
+  MainDataOK = (diff >= 0);
+  if (!MainDataOK) {
+    GST_DEBUG ("MainDataEnd: %d MainDataBegin: %d delta: %d",
+        tl->c_impl.main_data_end, III_side_info.main_data_begin, diff);
   }
 
   /* Copy the remaining main data in the bit reservoir to the start of the
    * huffman bit buffer, and then append the incoming bytes */
-  memmove (tl->c_impl.hb_buf, tl->c_impl.hb_buf +
-      tl->c_impl.main_data_end - III_side_info.main_data_begin,
-      III_side_info.main_data_begin);
-  tl->c_impl.main_data_end = III_side_info.main_data_begin;
-
+  if (MainDataOK) {
+    if (diff > 0) {
+      memmove (tl->c_impl.hb_buf, tl->c_impl.hb_buf + diff,
+          III_side_info.main_data_begin);
+      tl->c_impl.main_data_end = III_side_info.main_data_begin;
+    }
+  }
   /* And append the incoming bytes to the reservoir */
   bs_getbytes (tl->bs, tl->c_impl.hb_buf + tl->c_impl.main_data_end,
       hdr->main_slots);
   tl->c_impl.main_data_end += hdr->main_slots;
 
+  if (!MainDataOK) {
+    GST_DEBUG ("Bad frame - not enough main data bits");
+    return MP3TL_ERR_BAD_FRAME;
+  }
+
   /* And setup the huffman bitstream reader for this data */
   h_setbuf (bb, tl->c_impl.hb_buf, tl->c_impl.main_data_end);
 
@@ -1831,10 +1826,6 @@ c_decode_mp3 (mp3tl * tl)
 gboolean
 mp3_c_init (mp3tl * tl)
 {
-#ifdef USE_LIBOIL
-  oil_init ();
-#endif
-
   init_hybrid (&tl->c_impl);
   init_mdct ();
 
