@@ -363,7 +363,6 @@ mp3tl_gather_frame (mp3tl * tl, gint * _offset, gint * _length)
     remain = hdr->frame_bits - (bs_read_pos (tl->bs) - frame_start);
     if (hdr->frame_bits < (8 * MIN_FRAME_SIZE)) {
       GST_LOG ("Header indicates a frame too small to be correct");
-      valid = FALSE;
     } else if (bs_bits_avail (tl->bs) >= hdr->frame_bits) {
       guint32 sync_word;
       fr_header next_hdr;
@@ -563,6 +562,11 @@ mp3tl_decode_frame (mp3tl * tl, guint8 * samples, guint bufsize)
 #else
       ret = c_decode_mp3 (tl);
 #endif
+      if (ret == MP3TL_ERR_BAD_FRAME) {
+        /* Consume the data from our bitreader */
+        bs_consume (tl->bs, hdr->frame_bits - (SYNC_WORD_LNGTH + HEADER_LNGTH));
+        return ret;
+      }
 
       if (ret != MP3TL_ERR_OK)
         return ret;
